@@ -24,9 +24,12 @@ This module contains core API parsers.
 :license: Apache 2.0, see LICENSE for more details.
 
 """
+from __future__ import absolute_import, annotations
+
 import base64
 import json
 from abc import ABCMeta, abstractmethod
+from typing import Any, cast
 
 
 class Sse:
@@ -34,14 +37,14 @@ class Sse:
     __metaclass__ = ABCMeta
 
     @abstractmethod
-    def headers(self):
+    def headers(self) -> dict[str, str]:
         """Return headers."""
 
-    def tls_required(self):  # pylint: disable=no-self-use
+    def tls_required(self) -> bool:  # pylint: disable=no-self-use
         """Return TLS required to use this server-side encryption."""
         return True
 
-    def copy_headers(self):  # pylint: disable=no-self-use
+    def copy_headers(self) -> dict[str, str]:  # pylint: disable=no-self-use
         """Return copy headers."""
         return {}
 
@@ -49,7 +52,7 @@ class Sse:
 class SseCustomerKey(Sse):
     """ Server-side encryption - customer key type."""
 
-    def __init__(self, key):
+    def __init__(self, key: bytes):
         if len(key) != 32:
             raise ValueError(
                 "SSE-C keys need to be 256 bit base64 encoded",
@@ -57,13 +60,13 @@ class SseCustomerKey(Sse):
         b64key = base64.b64encode(key).decode()
         from .helpers import \
             md5sum_hash  # pylint: disable=import-outside-toplevel
-        md5key = md5sum_hash(key)
-        self._headers = {
+        md5key = cast(str, md5sum_hash(key))
+        self._headers: dict[str, str] = {
             "X-Amz-Server-Side-Encryption-Customer-Algorithm": "AES256",
             "X-Amz-Server-Side-Encryption-Customer-Key": b64key,
             "X-Amz-Server-Side-Encryption-Customer-Key-MD5": md5key,
         }
-        self._copy_headers = {
+        self._copy_headers: dict[str, str] = {
             "X-Amz-Copy-Source-Server-Side-Encryption-Customer-Algorithm":
             "AES256",
             "X-Amz-Copy-Source-Server-Side-Encryption-Customer-Key": b64key,
@@ -71,17 +74,17 @@ class SseCustomerKey(Sse):
             md5key,
         }
 
-    def headers(self):
+    def headers(self) -> dict[str, str]:
         return self._headers.copy()
 
-    def copy_headers(self):
+    def copy_headers(self) -> dict[str, str]:
         return self._copy_headers.copy()
 
 
 class SseKMS(Sse):
     """Server-side encryption - KMS type."""
 
-    def __init__(self, key, context):
+    def __init__(self, key: str, context: dict[str, Any]):
         self._headers = {
             "X-Amz-Server-Side-Encryption-Aws-Kms-Key-Id": key,
             "X-Amz-Server-Side-Encryption": "aws:kms"
@@ -92,17 +95,17 @@ class SseKMS(Sse):
                 base64.b64encode(data).decode()
             )
 
-    def headers(self):
+    def headers(self) -> dict[str, str]:
         return self._headers.copy()
 
 
 class SseS3(Sse):
     """Server-side encryption - S3 type."""
 
-    def headers(self):
+    def headers(self) -> dict[str, str]:
         return {
             "X-Amz-Server-Side-Encryption": "AES256"
         }
 
-    def tls_required(self):
+    def tls_required(self) -> bool:
         return False
